package org.apache.ibatis.builder.xml.dynamic;

import org.apache.ibatis.builder.SqlSourceBuilder;
import org.apache.ibatis.mapping.*;

import java.util.Map;

public class DynamicSqlSource implements SqlSource {

  private Configuration configuration;
  private SqlNode rootSqlNode;

  public DynamicSqlSource(Configuration configuration, SqlNode rootSqlNode) {
    this.configuration = configuration;
    this.rootSqlNode = rootSqlNode;
  }

  public BoundSql getBoundSql(Object parameterObject) {
    DynamicContext context = new DynamicContext(parameterObject);
    rootSqlNode.apply(context);
    SqlSourceBuilder sqlSourceParser = new SqlSourceBuilder(configuration);
    Class parameterType = parameterObject == null ? Object.class : parameterObject.getClass();
    SqlSource sqlSource = sqlSourceParser.parse(context.getSql(), parameterType);
    BoundSql boundSql = sqlSource.getBoundSql(parameterObject);
    for (Map.Entry<String,Object> entry : context.getBindings().entrySet()) {
      boundSql.setAdditionalParameter(entry.getKey(),entry.getValue());
    }
    return boundSql;
  }

}
